/* Dazuko Dummy. A dummy implementation to help porting to new platforms.
   Written by John Ogness <jogness@antivir.de>

   Copyright (c) 2004 H+BEDV Datentechnik GmbH
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   3. Neither the name of Dazuko nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
*/


#include <unistd.h>
#include <pthread.h>


#include "dazuko_dummy.h"
#include "dazuko_xp.h"
#include "dazukoio.h"

extern struct xp_atomic active;
extern int RUNNING;

#if defined(ON_OPEN_SUPPORT) || defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
static int dummy_dazuko_sys_open(const char *filename);
#endif

#if defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
static int dummy_dazuko_sys_close(const char *filename);
#endif

#ifdef ON_CLOSE_MODIFIED_SUPPORT
static int dummy_dazuko_sys_write(const char *filename);
#endif

#ifdef ON_EXEC_SUPPORT
static int dummy_dazuko_sys_execve(const char *filename);
#endif

#ifdef ON_UNLINK_SUPPORT
static int dummy_dazuko_sys_unlink(const char *filename);
#endif

#ifdef ON_RMDIR_SUPPORT
static int dummy_dazuko_sys_rmdir(const char *filename);
#endif


static int DUMMY_ID = 100;


/* mutex */

int xp_init_mutex(struct xp_mutex *mutex)
{
	mutex->mutex = 0;

	return 0;
}

int xp_down(struct xp_mutex *mutex)
{
	if (mutex->mutex > 0)
		printf("dazuko: xp_down(): we should block, but oh well...\n");

	mutex->mutex++;

	return 0;
}

int xp_up(struct xp_mutex *mutex)
{
	mutex->mutex--;

	return 0;
}

int xp_destroy_mutex(struct xp_mutex *mutex)
{
	return 0;
}


/* read-write lock */

int xp_init_rwlock(struct xp_rwlock *rwlock)
{
	rwlock->rwlock = 0;

	return 0;
}

int xp_write_lock(struct xp_rwlock *rwlock)
{
	if (rwlock->rwlock > 0)
		printf("dazuko: xp_write_lock(): we should block, but oh well...\n");

	rwlock->rwlock++;

	return 0;
}

int xp_write_unlock(struct xp_rwlock *rwlock)
{
	rwlock->rwlock--;

	return 0;
}

int xp_read_lock(struct xp_rwlock *rlock)
{
	if (rlock->rwlock > 0)
		printf("dazuko: xp_read_lock(): we should block, but oh well...\n");

	rlock->rwlock++;

	return 0;
}

int xp_read_unlock(struct xp_rwlock *rlock)
{
	rlock->rwlock--;

	return 0;
}

int xp_destroy_rwlock(struct xp_rwlock *rwlock)
{
	return 0;
}


/* wait-notify queue */

int xp_init_queue(struct xp_queue *queue)
{
	queue->queue = 0;

	return 0;
}

int xp_wait_until_condition(struct xp_queue *queue, int (*cfunction)(void *), void *cparam, int allow_interrupt)
{
	while (1)
	{
		if (cfunction(cparam))
			break;

		if (allow_interrupt && RUNNING == 0)
			return -1;

		sleep(1);
	}

	return 0;
}

int xp_notify(struct xp_queue *queue)
{
	return 0;
}

int xp_destroy_queue(struct xp_queue *queue)
{
	return 0;
}


/* memory */

void* xp_malloc(size_t size)
{
	return malloc(size);
}

int xp_free(void *ptr)
{
	free(ptr);

	return 0;
}

int xp_copyin(const void *user_src, void *kernel_dest, size_t size)
{
	memcpy(kernel_dest, user_src, size);

	return 0;
}

int xp_copyout(const void *kernel_src, void *user_dest, size_t size)
{
	memcpy(user_dest, kernel_src, size);

	return 0;
}

int xp_verify_user_writable(const void *user_ptr, size_t size)
{
	return 0;
}

int xp_verify_user_readable(const void *user_ptr, size_t size)
{
	return 0;
}


/* path attribute */

int xp_is_absolute_path(const char *path)
{
	return (path[0] == '/');
}


/* atomic */

int xp_atomic_set(struct xp_atomic *atomic, int value)
{
	atomic->atomic = value;

	return 0;
}

int xp_atomic_inc(struct xp_atomic *atomic)
{
	(atomic->atomic)++;

	return 0;
}

int xp_atomic_dec(struct xp_atomic *atomic)
{
	(atomic->atomic)--;

	return 0;
}

int xp_atomic_read(struct xp_atomic *atomic)
{
	return atomic->atomic;
}


/* file descriptor */

int xp_copy_file(struct xp_file *dest, struct xp_file *src)
{
	strncpy(dest->file, src->file, sizeof(dest->file)-1);
	dest->file[sizeof(dest->file)-1] = 0;

	return 0;
}

int xp_compare_file(struct xp_file *file1, struct xp_file *file2)
{
	return (strcmp(file1->file, file2->file));
}


/* file structure */

int xp_file_struct_check(struct dazuko_file_struct *dfs)
{
	int	length;

	length = strlen(dfs->extra_data->user_filename);

	dfs->filename = (char *)xp_malloc(length + 1);
	if (dfs->filename)
	{
		memcpy(dfs->filename, dfs->extra_data->user_filename, length);
		dfs->filename[length] = 0;
		dfs->filename_length = length;  /* the string length */
	}

	return 0;
}

int xp_file_struct_check_cleanup(struct dazuko_file_struct *dfs)
{
	return 0;
}

static int dazuko_file_struct_cleanup(struct dazuko_file_struct **dfs)
{
	if (dfs == NULL)
		return 0;

	if (*dfs == NULL)
		return 0;

	xp_file_struct_check_cleanup(*dfs);

	if ((*dfs)->filename)
		xp_free((*dfs)->filename);

	if ((*dfs)->extra_data)
		xp_free((*dfs)->extra_data);

	xp_free(*dfs);

	*dfs = NULL;

	return 0;
}


/* daemon id */

int xp_id_compare(struct xp_daemon_id *id1, struct xp_daemon_id *id2)
{
	if (id1 == NULL || id2 == NULL)
		return -1;

	if (id1->id == id2->id)
		return 0;

	return 1;
}

int xp_id_free(struct xp_daemon_id *id)
{
	xp_free(id);

	return 0;
}

struct xp_daemon_id* xp_id_copy(struct xp_daemon_id *id)
{
	struct xp_daemon_id	*ptr;

	if (id == NULL)
		return NULL;

	ptr = (struct xp_daemon_id *)xp_malloc(sizeof(struct xp_daemon_id));

	if (ptr != NULL)
		ptr->id = id->id;

	return ptr;
}


/* system calls */

#if defined(ON_OPEN_SUPPORT) || defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
static int dummy_dazuko_sys_open(const char *filename)
{
	struct dazuko_file_struct	*dfs = NULL;
	struct event_properties		event_p;
	struct xp_daemon_id		xp_id;
	int				error = 0;
	int				check_error = 0;
#if defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
	struct xp_file			file;
#endif

	xp_id.id = DUMMY_ID++;
	check_error = dazuko_sys_check(DAZUKO_ON_OPEN, 1, &xp_id);

	if (!check_error)
	{
		dazuko_bzero(&event_p, sizeof(event_p));

		event_p.flags = 10;
		event_p.set_flags = 1;
		event_p.mode = 11;
		event_p.set_mode = 1;
		event_p.pid = xp_id.id;
		event_p.set_pid = 1;
		event_p.uid = 12;
		event_p.set_uid = 1;

		dfs = (struct dazuko_file_struct *)xp_malloc(sizeof(struct dazuko_file_struct));
		if (dfs)
		{
			dazuko_bzero(dfs, sizeof(struct dazuko_file_struct));

			dfs->extra_data = (struct xp_file_struct *)xp_malloc(sizeof(struct xp_file_struct));
			if (dfs->extra_data)
			{
				dazuko_bzero(dfs->extra_data, sizeof(struct xp_file_struct));

				dfs->extra_data->user_filename = filename;

				error = dazuko_sys_pre(DAZUKO_ON_OPEN, dfs, &event_p);
			}
			else
			{
				xp_free(dfs);
				dfs = NULL;
			}
		}
	}

	if (error)
	{
		/* access should be blocked */

		error = XP_ERROR_PERMISSION;
	}
	else
	{
		/* call the standard open function */
		printf("dazuko: calling sys_open(\"%s\")\n", filename);

		/* error should be set to true if sys_open() failed */
		error = 0;

		#if defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
		{
			if (!check_error)
			{
				if (error)
				{
					check_error = -1;
				}
				else
				{
					/* normally this would be a pointer to an internal file structure */
					strncpy(file.file, filename, sizeof(file.file)-1);
				}
			}
		}
		#endif
	}

	if (!check_error)
	{
		#if defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
		{
			if (!error && dfs)
				dazuko_sys_post(DAZUKO_ON_OPEN, dfs, &file, &event_p);
		}
		#endif
	}

	dazuko_file_struct_cleanup(&dfs);

	return error;
}
#endif

#if defined(ON_CLOSE_SUPPORT) || defined(ON_CLOSE_MODIFIED_SUPPORT)
static int dummy_dazuko_sys_close(const char *filename)
{
	/* The kernel wants to close the given file
	 * descriptor. */

	int			error = 0;
	int			check_error = 0;
	struct xp_file		file;
	struct event_properties	event_p;
	struct xp_daemon_id	xp_id;

	xp_id.id = DUMMY_ID++;
	check_error = dazuko_sys_check(DAZUKO_ON_CLOSE, 1, &xp_id);

	if (!check_error)
	{
		/* normally this would be a pointer to an internal file structure */
		strncpy(file.file, filename, sizeof(file.file)-1);
	}

	/* call the standard close function */
	printf("dazuko: calling sys_close(\"%s\")\n", filename);

	/* error should be set to true if sys_close() failed */
	error = 0;

	if (!error && !check_error)
	{
		dazuko_bzero(&event_p, sizeof(event_p));
		event_p.pid = xp_id.id;
		event_p.set_pid = 1;
		event_p.uid = 14;
		event_p.set_uid = 1;

		dazuko_sys_post(DAZUKO_ON_CLOSE, NULL, &file, &event_p);
	}

	return error;
}
#endif

#ifdef ON_CLOSE_MODIFIED_SUPPORT
static int dummy_dazuko_sys_write(const char *filename)
{
	/* The kernel wants to write to the given file
	 * descriptor. */

	int			error = 0;
	int			check_error = 0;
	struct xp_file		file;
	struct event_properties	event_p;
	struct xp_daemon_id	xp_id;

	xp_id.id = DUMMY_ID++;
	check_error = dazuko_sys_check(DAZUKO_ON_CLOSE_MODIFIED, 1, &xp_id);

	if (!check_error)
	{
		/* normally this would be a pointer to an internal file structure */
		strncpy(file.file, filename, sizeof(file.file)-1);
	}

	/* call the standard write function */
	printf("dazuko: calling sys_write(\"%s\")\n", filename);

	/* error should be set to true if sys_write() failed */
	error = 0;
	
	if (!error && !check_error)
	{
		dazuko_bzero(&event_p, sizeof(event_p));

		dazuko_sys_post(DAZUKO_ON_CLOSE_MODIFIED, NULL, &file, &event_p);
	}

	return error;
}
#endif

static int dummy_dazuko_sys_generic(int event, const char *filename, int daemon_is_allowed)
{
	struct dazuko_file_struct	*dfs = NULL;
	struct event_properties		event_p;
	struct xp_daemon_id		xp_id;
	int				error = 0;
	int				check_error = 0;

	xp_id.id = DUMMY_ID++;
	check_error = dazuko_sys_check(event, daemon_is_allowed, &xp_id);

	if (check_error == 0)
	{
		dazuko_bzero(&event_p, sizeof(event_p));
		event_p.pid = xp_id.id;
		event_p.set_pid = 1;
		event_p.uid = 15;
		event_p.set_uid = 1;

		dfs = (struct dazuko_file_struct *)xp_malloc(sizeof(struct dazuko_file_struct));
		if (dfs)
		{
			dazuko_bzero(dfs, sizeof(struct dazuko_file_struct));

			dfs->extra_data = (struct xp_file_struct *)xp_malloc(sizeof(struct xp_file_struct));
			if (dfs->extra_data)
			{
				dazuko_bzero(dfs->extra_data, sizeof(struct xp_file_struct));

				dfs->extra_data->user_filename = filename;

				error = dazuko_sys_pre(event, dfs, &event_p);

				dazuko_file_struct_cleanup(&dfs);
			}
			else
			{
				xp_free(dfs);
				dfs = NULL;
			}
		}
	}

	return error;
}

#ifdef ON_EXEC_SUPPORT
static int dummy_dazuko_sys_execve(const char *filename)
{
	int	error;

	error = dummy_dazuko_sys_generic(DAZUKO_ON_EXEC, filename, 0);

	if (error)
		return error;

	/* call the standard exec function */
	printf("dazuko: calling sys_exec(\"%s\")\n", filename);

	/* return the return value of the standard exec function */
	return 0;
}
#endif

#ifdef ON_UNLINK_SUPPORT
static int dummy_dazuko_sys_unlink(const char *filename)
{
	int	error;

	error = dummy_dazuko_sys_generic(DAZUKO_ON_UNLINK, filename, 1);

	if (error)
		return error;

	/* call the standard unlink function */
	printf("dazuko: calling sys_unlink(\"%s\")\n", filename);

	/* return the return value of the standard unlink function */
	return 0;
}
#endif

#ifdef ON_RMDIR_SUPPORT
static int dummy_dazuko_sys_rmdir(const char *filename)
{
	int	error;

	error = dummy_dazuko_sys_generic(DAZUKO_ON_RMDIR, filename, 1);

	if (error)
		return error;

	/* call the standard rmdir function */
	printf("dazuko: calling sys_rmdir(\"%s\")\n", filename);

	/* return the return value of the standard rmdir function */
	return 0;
}
#endif


/* system hook */

int xp_sys_hook()
{
	printf("dazuko: setting up communication for daemons\n");

	printf("dazuko: hooking file access calls\n");

	return 0;
}

int xp_sys_unhook()
{
	printf("dazuko: unhooking file access calls\n");

	printf("dazuko: closing communication for daemons\n");

	return 0;
}


/* output */

int xp_print(const char *fmt, ...)
{
	va_list	ap;

	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);

	return 0;
}


/* init/exit */

/* this function represents entering the kernel:
 * the userland request pointer is passed to the kernel as a string
 */
int dummy_process_request(const char *buffer, size_t buffer_size)
{
	struct dazuko_request *request;
	char *value;
	struct xp_daemon_id xp_id;

	if (dazuko_get_value("\nRA=", buffer, &value) != 0)
	{
		xp_print("dazuko: error: dummy_dazuko_device_write.RA missing\n");
		return XP_ERROR_FAULT;
	}

	request = (struct dazuko_request *)dazuko_strtol(value);

	xp_free(value);

	xp_id.id = 7;

	if (dazuko_handle_user_request(request, &xp_id) != 0)
		return -1;

	return 0;
}

/* used for generating access events from the keyboard */
int readinput()
{
	char line[256];
	char *p;

	if (fgets(line, sizeof(line), stdin) == NULL)
		return -1;

	p = strchr(line, 10);
	if (p != NULL)
		*p = 0;

	p = strchr(line, 13);
	if (p != NULL)
		*p = 0;

	if (strncmp(line, "open ", 5) == 0)
	{
		dummy_dazuko_sys_open(line + 5);
	}
	else if (strncmp(line, "write ", 6) == 0)
	{
		dummy_dazuko_sys_write(line + 6);
	}
	else if (strncmp(line, "close ", 6) == 0)
	{
		dummy_dazuko_sys_close(line + 6);
	}
	else if (strncmp(line, "exec ", 5) == 0)
	{
		dummy_dazuko_sys_execve(line + 5);
	}
	else if (strncmp(line, "unlink ", 7) == 0)
	{
		dummy_dazuko_sys_unlink(line + 7);
	}
	else if (strncmp(line, "rmdir ", 6) == 0)
	{
		dummy_dazuko_sys_rmdir(line + 6);
	}
	else if (strcmp(line, "quit") == 0)
	{
		RUNNING = 0;

		return -1;
	}
	else
	{
		printf("usage:\n");
		printf("    open <filename>\n");
		printf("    write <filename>\n");
		printf("    close <filename>\n");
		printf("    exec <filename>\n");
		printf("    unlink <filename>\n");
		printf("    rmdir <filename>\n");
		printf("    quit\n");
	}

	return 0;
}

void* dummy_main(void *param);

int main()
{
	pthread_t id;

	if (dazuko_init() == 0)
	{
		/* start the "userspace" daemon */
		pthread_create(&id, NULL, dummy_main, NULL);

		while (1)
		{
			/* read from keyboard */
			if (readinput() != 0)
			{
				pthread_join(id, NULL);

				sleep(1);

				if (dazuko_exit() == 0)
					break;
			}
		}
	}

	return 0;
}

