/* Example program demonstrating the capabilities/interface of Dazuko.
   Written by John Ogness <jogness@antivir.de>

   Copyright (c) 2004 H+BEDV Datentechnik GmbH
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

   3. Neither the name of Dazuko nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "dazukoio.h"

int RUNNING = 1;

void print_access(struct dazuko_access *acc)
{
	/* print access data */

	if (!acc->set_event)
		return;

	printf("userspace daemon --> ");

	switch (acc->event)
	{
		case DAZUKO_ON_OPEN:
			printf("OPEN  ");
			break;
		case DAZUKO_ON_CLOSE:
			printf("CLOSE ");
			break;
		case DAZUKO_ON_CLOSE_MODIFIED:
			printf("CLOSE (modified)");
			break;
		case DAZUKO_ON_EXEC:
			printf("EXEC  ");
			break;
		case DAZUKO_ON_UNLINK:
			printf("UNLINK");
			break;
		case DAZUKO_ON_RMDIR:
			printf("RMDIR ");
			break;
		default:
			printf("????   event:%d ", acc->event);
			break;
	}

	if (acc->set_uid)
		printf(" uid:%d", acc->uid);

	if (acc->set_pid)
		printf(" pid:%d", acc->pid);

	if (acc->set_mode)
		printf(" mode:%d", acc->mode);

	if (acc->set_flags)
		printf(" flags:%d", acc->flags);

	if (acc->set_file_uid)
		printf(" file_uid:%d", acc->file_uid);

	if (acc->set_file_gid)
		printf(" file_gid:%d", acc->file_gid);

	if (acc->set_file_mode)
		printf(" file_mode:%d", acc->file_mode);

	if (acc->set_file_device)
		printf(" file_device:%d", acc->file_device);

	if (acc->set_file_size)
		printf(" file_size:%lu", acc->file_size);

	if (acc->set_filename)
		printf(" file:%s", acc->filename);

	printf("\n");

	fflush(stdout);
}

void* dummy_main(void *param)
{
	struct dazuko_access *access = NULL;

	sleep(1);

	dazukoRegister("DummyDaemon", "RW");
	dazukoSetAccessMask(63);
	dazukoAddIncludePath("/tmp/");

	while (RUNNING)
	{
		if (dazukoGetAccess(&access) != 0)
			break;

		print_access(access);

		if (strcmp(access->filename, "/tmp/deny.txt") == 0)
			access->deny = 1;

		dazukoReturnAccess(&access);
	}

	dazukoUnregister();

	return NULL;
}

/* the real main() is in ../dazuko_dummy.c */

